from . import RunDB


class BaseDB(RunDB):

    def __init__(self, driver, conn_str):

        self.name = 'BaseDB'
        self.conn_str = conn_str
        self.driver = driver(conn_str)

    def __str__(self):
        return self.driver.__str__()

    @property
    def schema(self):
        return self.driver.schema

    @schema.setter
    def schema(self, schema):
        self.driver.schema = schema

    def init_table(self, table):
        return self.driver.init_table(table)

    def update_record(self, table, run_id, key, value):
        return self.driver.update_record(table, run_id, key, value)

    def get_record(self, table, action='up', run_id=None):
        return self.driver.get_record(table, run_id=run_id)

    def get_records(self, table=[], count=10):
        return self.driver.get_records(table=table, count=count)

    def get_tables(self):
        return self.driver.get_tables()

    def remove_record(self, table, key, value):
        return self.driver.remove_record(table, key, value)

    def search(self, table, key):
        return self.driver.search(table, key)

    def query(self, table, query_info):
        return self.driver.query(table, query_info)

    def purge(self, table=None):
        return self.driver.purge(table)



#!/usr/bin/env python

import yaml

from linchpin.exceptions import LinchpinError


def yaml2json(pf):

    """ parses yaml file into json object """

    with open(pf, 'r') as stream:
        try:
            pf = yaml.load(stream)
            return pf
        except yaml.YAMLError as exc:
            raise LinchpinError(exc)



#!/usr/bin/env python

import os
import re
import ast
import yaml
import json
import time
import hashlib

from collections import OrderedDict
from uuid import getnode as get_mac

from linchpin.api.ansible_runner import ansible_runner
from linchpin.api.utils import yaml2json
from linchpin.api.fetch import FETCH_CLASS

from linchpin.hooks.state import State
from linchpin.hooks import LinchpinHooks

from linchpin.rundb.basedb import BaseDB
from linchpin.rundb.drivers import DB_DRIVERS

from linchpin.exceptions import LinchpinError


class LinchpinAPI(object):

    def __init__(self, ctx):
        """
        LinchpinAPI constructor

        :param ctx: context object from api/context.py

        """

        self.ctx = ctx
        base_path = '/'.join(os.path.dirname(__file__).split('/')[0:-2])
        pkg = self.get_cfg(section='lp', key='pkg', default='linchpin')
        self.lp_path = '{0}/{1}'.format(base_path, pkg)

        self.set_evar('from_api', True)

        self.hook_state = None
        self._hook_observers = []
        self.playbook_pre_states = self.get_cfg('playbook_pre_states',
                                                {'up': 'preup',
                                                 'destroy': 'predestroy'})
        self.playbook_post_states = self.get_cfg('playbook_post_states',
                                                 {'up': 'postup',
                                                  'destroy': 'postdestroy',
                                                  'postinv': 'inventory'})
        self.hooks = LinchpinHooks(self)
        self.target_data = {}

        if not self.workspace:
            self.workspace = os.path.realpath(os.path.curdir)


    def get_cfg(self, section=None, key=None, default=None):
        """
        Get cfgs value(s) by section and/or key, or the whole cfgs object

        :param section: section from ini-style config file

        :param key: key to get from config file, within section

        :param default: default value to return if nothing is found.

        Does not apply if section is not provided.
        """

        return self.ctx.get_cfg(section=section, key=key, default=default)


    def set_cfg(self, section, key, value):
        """
        Set a value in cfgs. Does not persist into a file,
        only during the current execution.


        :param section: section within ini-style config file

        :param key: key to use

        :param value: value to set into section within config file
        """

        self.ctx.set_cfg(section, key, value)


    def get_evar(self, key=None, default=None):
        """
        Get the current evars (extra_vars)

        :param key: key to use

        :param default: default value to return if nothing is found
        (default: None)
        """

        return self.ctx.get_evar(key=key, default=default)


    def set_evar(self, key, value):
        """
        Set a value into evars (extra_vars). Does not persist into a file,
        only during the current execution.

        :param key: key to use

        :param value: value to set into evars
        """

        self.ctx.set_evar(key, value)


    @property
    def pinfile(self):
        """
        getter function for pinfile name
        """

        return self.ctx.pinfile


    @pinfile.setter
    def pinfile(self, pinfile):
        """
        setter for pinfile name
        """

        self.ctx.pinfile = pinfile


    @property
    def workspace(self):
        """
        getter function for context workspace
        """

        return self.ctx.workspace


    @workspace.setter
    def workspace(self, workspace):
        """
        setter for context workspace
        """

        self.ctx.workspace = workspace


    @property
    def hook_state(self):
        """
        getter function for hook_state property of the API object
        """

        return self.hook_state


    @hook_state.setter
    def hook_state(self, hook_state):
        """
        hook_state property setter , splits the hook_state string in
        subhook_state and sets linchpin.hook_state object

        :param hook_state: valid hook_state string mentioned in linchpin.conf
        """

        # call run_hooks after hook_state is being set
        if hook_state is None:
            return
        else:
            self.ctx.log_debug('hook {0} initiated'.format(hook_state))
            self._hook_state = State(hook_state, None, self.ctx)

            for callback in self._hook_observers:
                callback(self._hook_state)


    def bind_to_hook_state(self, callback):
        """
        Function used by LinchpinHooksclass to add callbacks

        :param callback: callback function
        """

        self._hook_observers.append(callback)


    def set_magic_vars(self, uhash):
        """
        Function inbuilt to set magic vars for ansible context
        """

        try:
            t_f = open(self.get_evar("topology"), "r").read()
            t_f = yaml.load(t_f)
            topology_name = t_f["topology_name"]
        except Exception as e:
            self.ctx.log_info("{0}".format(str(e)))
            topology_name = self.get_evar("topology").split("/")[-1]
            # defaults to file name if there is any error
            topology_name = topology_name.split(".")[-2]

        inv_folder = str(self.get_evar('inventories_folder'))

        inv_file = '{0}/{1}/{2}-{3}{4}'.format(self.workspace,
                                               inv_folder,
                                               topology_name,
                                               uhash,
                                               self.get_cfg('extensions',
                                                            'inventory',
                                                            'inventory'))

        self.set_evar('inventory_file', inv_file)
        self.set_evar('topology_name', topology_name)


    def lp_rise(self, pinfile, targets='all', run_id=None):
        """
        DEPRECATED

        An alias for lp_up. Used only for backward compatibility.
        """

        self.lp_up(pinfile, targets, run_id=run_id)


    def lp_up(self, pinfile, targets='all', run_id=None):
        """
        This function takes a list of targets, and provisions them according
        to their topology. If an layout argument is provided, an inventory
        will be generated for the provisioned nodes.

        :param pinfile:
            Provided PinFile, with available targets,

        :param targets:
            A tuple of targets to provision.
        """

        return self.run_playbook(pinfile, targets, action="up", run_id=run_id)


    def lp_drop(self, pinfile, targets, run_id=None):
        """
        DEPRECATED

        An alias for lp_destroy. Used only for backward compatibility.
        """

        return self.lp_destroy(pinfile, targets, run_id=run_id)


    def lp_destroy(self, pinfile, targets='all', run_id=None):
        """
        This function takes a list of targets, and performs a destructive
        teardown, including undefining nodes, according to the target.

        .. seealso:: lp_down - currently unimplemented

        :param pinfile:
            Provided PinFile, with available targets,

        :param targets:
            A tuple of targets to destroy.
        """

        return self.run_playbook(pinfile,
                                 targets,
                                 action="destroy",
                                 run_id=run_id)


    def lp_down(self, pinfile, targets='all'):
        """
        This function takes a list of targets, and performs a shutdown on
        nodes in the target's topology. Only providers which support shutdown
        from their API (Ansible) will support this option.

        CURRENTLY UNIMPLEMENTED

        .. seealso:: lp_destroy

        :param pinfile:
            Provided PinFile, with available targets,

        :param targets:
            A tuple of targets to provision.
        """

        pass


    def lp_fetch(self, src, fetch_type, root):
        if root is not None:
            root = list(filter(None, root.split(',')))

        dest = self.ctx.workspace
        if not os.path.exists(dest):
            raise LinchpinError(dest + " does not exist")

        fetch_aliases = {
            "topologies": self.get_evar("topologies_folder"),
            "layouts": self.get_evar("layouts_folder"),
            "resources": self.get_evar("resources_folder"),
            "hooks": self.get_evar("hooks_folder"),
            "workspace": "workspace"
        }

        fetch_dir = fetch_aliases.get(fetch_type, "workspace")


        cache_path = os.path.abspath(os.path.join(os.path.expanduser('~'),
                                                  '.cache/linchpin'))
        if not os.path.exists(cache_path):
            os.mkdir(cache_path)

        protocol_regex = OrderedDict([
            ('((git|ssh|http(s)?)|(git@[\w\.]+))'
                '(:(//)?)([\w\.@\:/\-~]+)(\.git)(/)?',
                'FetchGit'),
            ('^(http|https)://', 'FetchHttp'),
            ('^(file)://', 'FetchLocal')
        ])
        fetch_protocol = None
        for regex, obj in protocol_regex.items():
            if re.match(regex, src):
                fetch_protocol = obj
                break
        if fetch_protocol is None:
            raise LinchpinError("The protocol speficied is not supported")


        fetch_class = FETCH_CLASS[fetch_protocol](self.ctx, fetch_dir, src,
                                                  dest, cache_path, root)
        fetch_class.fetch_files()

        fetch_class.copy_files()


    def lp_journal(self, targets=[], fields=None, count=None):

        rundb = self.setup_rundb()

        journal = {}

        if not len(targets):
            targets = rundb.get_tables()


        for target in targets:
            journal[target] = rundb.get_records(table=target, count=count)

        return journal


    def find_topology(self, topology):
        """
        Find the topology to be acted upon. This could be pulled from a
        registry.

        :param topology:
            name of topology from PinFile to be loaded

        """

        topo_path = os.path.realpath('{0}/{1}'.format(
                                     self.ctx.workspace,
                                     self.get_evar('topologies_folder',
                                                   'topologies')))

        topos = os.listdir(topo_path)

        if topology in topos:
            return os.path.realpath('{0}/{1}'.format(topo_path, topology))

        raise LinchpinError('Topology {0} not found in'
                            ' workspace'.format(topology))


    def setup_rundb(self):
        """
        Configures the run database parameters, sets them into extra_vars
        """

        rundb_conn_default = '~/.config/linchpin/rundb-::mac::.json'
        rundb_conn = self.get_cfg(section='lp',
                                  key='rundb_conn',
                                  default=rundb_conn_default)
        rundb_type = self.get_cfg(section='lp',
                                  key='rundb_type',
                                  default='TinyRunDB')
        rundb_conn_type = self.get_cfg(section='lp',
                                       key='rundb_conn_type',
                                       default='file')
        self.rundb_hash = self.get_cfg(section='lp',
                                       key='rundb_hash',
                                       default='sha256')

        if rundb_conn_type == 'file':
            rundb_conn_int = rundb_conn.replace('::mac::', str(get_mac()))
            rundb_conn_int = os.path.expanduser(rundb_conn_int)
            rundb_conn_dir = os.path.dirname(rundb_conn_int)

            if not os.path.exists(rundb_conn_dir):
                os.mkdir(rundb_conn_dir)


        self.set_evar('rundb_type', rundb_type)
        self.set_evar('rundb_conn', rundb_conn_int)
        self.set_evar('rundb_hash', self.rundb_hash)


        return BaseDB(DB_DRIVERS[rundb_type], rundb_conn_int)


    def run_playbook(self, pinfile, targets=[], action='up', run_id=None):
        """
        This function takes a list of targets, and executes the given
        action (up, destroy, etc.) for each provided target.

        :param pinfile: Provided PinFile, with available targets,

        :param targets: A tuple of targets to run. (default: 'all')

        .. .note:: The `run_id` value differs from the `rundb_id`, in that
                   the `run_id` is an existing value in the database.
                   The `rundb_id` value is created to store the new record.
                   If the `run_id` is passed, it is used to collect an existing
                   `uhash` value from the given `run_id`, which is in turn used
                   to perform an idempotent reprovision, or destroy provisioned
                   resources.
        """

        pf = yaml2json(pinfile)

        # playbooks check whether from_api is defined
        self.ctx.log_debug('from_api: {0}'.format(self.get_evar('from_api')))

        # playbooks check whether from_cli is defined
        # if not, vars get loaded from linchpin.conf
        self.set_evar('from_api', True)
        self.set_evar('lp_path', self.lp_path)

        # do we display the ansible output to the console?
        ansible_console = False
        if self.ctx.cfgs.get('ansible'):
            ansible_console = (
                ast.literal_eval(self.ctx.cfgs['ansible'].get('console',
                                                              'False')))

        if not ansible_console:
            ansible_console = self.ctx.verbose

        self.set_evar('default_resources_path', '{0}/{1}'.format(
                      self.ctx.workspace,
                      self.get_evar('resources_folder',
                                    default='resources')))

        # playbooks still use this var, keep it here
        self.set_evar('default_inventories_path',
                      '{0}/{1}'.format(self.ctx.workspace,
                                       self.get_evar('inventories_folder',
                                                     default='inventories')))

        # add this because of magic_var evaluation in ansible
        self.set_evar('inventory_dir', self.get_evar(
                      'default_inventories_path',
                      default='inventories'))

        self.set_evar('state', 'present')


        if action == 'destroy':
            self.set_evar('state', 'absent')

        results = {}

        # determine what targets is equal to
        if (set(targets) ==
                set(pf.keys()).intersection(targets) and len(targets) > 0):
            pass
        elif len(targets) == 0:
            targets = set(pf.keys()).difference()
        else:
            raise LinchpinError("One or more invalid targets found")

        # create localhost file in workspace for user if it doesn't exist
        if not os.path.exists('{0}/localhost'.format(self.workspace)):
            with open('{0}/localhost'.format(self.workspace), 'w') as f:
                f.write('localhost\n')

        for target in targets:

            results[target] = {}

            # initialize rundb table
            dateformat = self.get_cfg('logger',
                                      'dateformat',
                                      default='%m/%d/%Y %I:%M:%S %p')

            self.set_evar('target', target)

            rundb = self.setup_rundb()

            rundb_schema = json.loads(self.get_cfg(section='lp',
                                      key='rundb_schema'))
            rundb.schema = rundb_schema
            self.set_evar('rundb_schema', rundb_schema)

            start = time.strftime(dateformat)
            uhash = None

            # generate a new rundb_id
            # (don't confuse it with an already existing run_id)
            rundb_id = rundb.init_table(target)

            if action == 'up' and not run_id:
                uh = hashlib.new(self.rundb_hash,
                                 ':'.join([target, str(rundb_id), start]))
                uhash = uh.hexdigest()[-4:]
            elif action == 'destroy' or run_id:
                # look for the action='up' records to destroy
                data, orig_run_id = rundb.get_record(target,
                                                     action='up',
                                                     run_id=run_id)

                if data:
                    self.set_evar('orig_run_id', orig_run_id)
                    uhash = data.get('uhash')
                    self.ctx.log_debug("using data from"
                                       " run_id: {0}".format(run_id))
                else:
                    raise LinchpinError("Attempting to perform '{0}' action on"
                                        " target: '{1}' failed. No records"
                                        " available.".format(action, target))
            else:
                raise LinchpinError("run_id '{0}' does not match any existing"
                                    " records".format(run_id))


            self.ctx.log_debug('rundb_id: {0}'.format(rundb_id))
            self.ctx.log_debug('uhash: {0}'.format(uhash))
            rundb.update_record(target, rundb_id, 'uhash', uhash)

            rundb.update_record(target, rundb_id, 'start', start)
            rundb.update_record(target, rundb_id, 'action', action)

            self.set_evar('_action', action)
            self.set_evar('rundb_id', rundb_id)
            self.set_evar('uhash', uhash)

            self.set_evar('topology', self.find_topology(
                          pf[target]["topology"]))

            rundb.update_record(target,
                                rundb_id,
                                'inputs',
                                [
                                    {'topology_file': pf[target]['topology']}
                                ])

            if 'layout' in pf[target]:
                self.set_evar('layout_file',
                              '{0}/{1}/{2}'.format(self.ctx.workspace,
                                                   self.get_evar(
                                                       'layouts_folder'),
                                                   pf[target]["layout"]))

                ws = self.ctx.workspace
                layout_folder = self.get_evar("layouts_folder",
                                              default='layouts')
                layout_file = pf[target]['layout']
                layout_path = '{0}/{1}/{2}'.format(ws,
                                                   layout_folder,
                                                   layout_file)
                layout_data = yaml2json(layout_path)

                rundb.update_record(target,
                                    rundb_id,
                                    'inputs',
                                    [
                                        {'layout_file': layout_file}
                                    ])

                rundb.update_record(target,
                                    rundb_id,
                                    'inputs',
                                    [
                                        {'layout_data': layout_data}
                                    ])

            # parse topology_file and set inventory_file
            self.set_magic_vars(uhash)

            # set the current target data
            self.target_data = pf[target]
            self.target_data["extra_vars"] = self.get_evar()

            # note : changing the state triggers the hooks
            self.hooks.rundb = (rundb, rundb_id)
            self.pb_hooks = self.get_cfg('hookstates', action)
            self.ctx.log_debug('calling: {0}{1}'.format('pre', action))

            if 'pre' in self.pb_hooks:
                self.hook_state = '{0}{1}'.format('pre', action)

            # FIXME need to add rundb data for hooks results

            # invoke the appropriate action
            return_code, results[target]['task_results'] = (
                self._invoke_playbook(action=action,
                                      console=ansible_console)
            )

            if not return_code:
                self.ctx.log_state("Action '{0}' on Target '{1}' is "
                                   "complete".format(action, target))

            # FIXME Check the result[target] value here, and fail if applicable.
            # It's possible that a flag might allow more targets to run, then
            # return an error code at the end.

            # add post provision hook for inventory generation
            if 'inv' in self.pb_hooks:
                self.hook_state = 'postinv'

            if 'post' in self.pb_hooks:
                self.hook_state = '{0}{1}'.format('post', action)

            end = time.strftime(dateformat)
            rundb.update_record(target, rundb_id, 'end', end)
            rundb.update_record(target, rundb_id, 'rc', return_code)

            run_data = rundb.get_record(target, action=action, run_id=rundb_id)
            results[target]['rundb_data'] = {rundb_id: run_data[0]}

        return (return_code, results)


    def _invoke_playbook(self, action='up', console=True):
        """
        Uses the Ansible API code to invoke the specified linchpin playbook

        :param action: Which ansible action to run (default: 'up')
        :param console: Whether to display the ansible console (default: True)
        """

        pb_path = '{0}/{1}'.format(self.lp_path,
                                   self.ctx.get_evar('playbooks_folder',
                                                     'provision'))
        module_path = '{0}/{1}/'.format(pb_path, self.get_cfg('lp',
                                                              'module_folder',
                                                              'library'))
        playbook_path = '{0}/{1}'.format(pb_path, self.get_cfg('playbooks',
                                                               action,
                                                               'site.yml'))
        extra_var = self.get_evar()

        return ansible_runner(playbook_path,
                              module_path,
                              extra_var,
                              console=console)



import os
import shutil
try:
    import configparser
except ImportError:
        import ConfigParser as configparser


from abc import ABCMeta, abstractmethod
from linchpin.exceptions import LinchpinError


class Fetch(object):
    __metaclass__ = ABCMeta

    def __init__(self, ctx, fetch_type, dest, root):
        self.ctx = ctx
        self.fetch_type = fetch_type
        self.root = root
        self.tempdirs = []
        self.dest = os.path.abspath(os.path.realpath(dest))
        self.config_path = os.path.abspath(os.path.join(os.path.expanduser('~'),
                                           '.cache/linchpin/fetch.conf'))
        self.cfgs = self.read_cfg()

    @abstractmethod
    def fetch_files(self):
        pass

    def copy_files(self):
        if self.fetch_type == 'workspace':
            for path in self.tempdirs:
                self.copy_dir(path, self.dest)
        else:
            self.transfer_section(self.fetch_type)


    def transfer_section(self, section):
        dest_dir = os.path.join(self.dest, section)
        dir_exists = True
        if section not in os.listdir(self.dest):
            dir_exists = False
            os.mkdir(dest_dir)

        for path in self.tempdirs:
            src_dir = os.path.join(path, section)
            if not os.path.exists(src_dir):
                if not dir_exists:
                    shutil.rmtree(dest_dir)
                raise LinchpinError('The {0} directory does not exist in '
                                    '{1}'.format(self.fetch_type, self.src))
            self.copy_dir(src_dir, dest_dir)

    def copy_dir(self, src, dest):
        for root, dirs, files in os.walk(src):
            files = [f for f in files if not f[0] == '.']
            dirs[:] = [d for d in dirs if not d[0] == '.']
            if not os.path.isdir(root):
                os.makedirs(root)
            for file in files:
                rel_path = root.replace(src, '').lstrip(os.sep)
                dest_path = os.path.join(dest, rel_path)

                if not os.path.isdir(dest_path):
                    try:
                        os.makedirs(dest_path)
                    except OSError as e:
                        if e.errno == 17:
                            raise LinchpinError('File {0} already exists'
                                                ' in destination directory'
                                                .format(e.filename))

                s = os.path.join(root, file)
                d = os.path.join(dest_path, file)

                if (not os.path.exists(d)) or (os.stat(d).st_mtime -
                                               os.stat(dest).st_mtime > 1):
                    try:
                        if os.path.islink(s) and os.path.exists(os.readlink(s)):
                            linkto = os.readlink(s)
                            os.symlink(linkto, d)
                        else:
                            shutil.copy2(s, d)
                    except (IOError, OSError):
                        pass


    def read_cfg(self):
        config = configparser.ConfigParser()
        config.optionxform = str

        cfgs = {}
        if not os.path.exists(self.config_path):
            config.add_section('http')
            config.add_section('git')
            config.add_section('local')
            with open(self.config_path, 'w') as configfile:
                config.write(configfile)
        else:
            config.read(self.config_path)

        for section in config.sections():
            cfgs[section] = {}
            if config.items(section) is None:
                continue
            for k, v in config.items(section):
                cfgs[section][k] = v
        return cfgs

    def write_cfg(self, section, key, value):
        config = configparser.ConfigParser()
        config.optionxform = str
        config.read(self.config_path)
        config.set(section, key, value)

        with open(self.config_path, 'w') as configfile:
            config.write(configfile)



#!/usr/bin/env python
import os
import sys


filepath = os.path.realpath(__file__)
filepath = "/".join(filepath.split("/")[0:-2])
sys.path.append(filepath)

from InventoryFilters import GCloudInventory  # noqa


class FilterModule(object):
    ''' A filter to fix interface's name format '''
    def filters(self):
        inv = GCloudInventory.GCloudInventory()
        return {
            'gcloud_inv': inv.get_inventory
        }



#!/usr/bin/env python

import StringIO
from camel import Camel

from InventoryFilter import InventoryFilter
from InventoryProviders import get_all_drivers


class GenericInventory(InventoryFilter):

    def __init__(self):
        InventoryFilter.__init__(self)
        self.filter_classes = get_all_drivers()

    def get_host_ips(self, topo):
        """
        currently it will return the dict as follows:
        {
        "aws_inv": ["x.x.x.x"],
        "os_inv" :["x.x.x.x","x.x.x.x"],
        "gcloud_inv" : ["x.x.x.x","x.x.x.x"]
        }
        """
        host_ip_dict = {}
        for inv_filter in self.filter_classes:
            ips = self.filter_classes[inv_filter]().get_host_ips(topo)
            host_ip_dict[inv_filter] = ips
        return host_ip_dict

    def get_hosts_by_count(self, host_dict, count):
        """
        currently this function gets all the ips/hostname according to the
        order in which inventories are specified. later can be modified
        to work with user input
        """
        all_hosts = []
        for inv in host_dict:
            all_hosts.extend(host_dict[inv])
        return all_hosts[:count]

    def get_inventory(self, topo, layout):
        layout = Camel().load(layout)
        # get all the topology host_ips
        host_ip_dict = self.get_host_ips(topo)
        # get the count of all layout hosts needed
        layout_host_count = self.get_layout_hosts(layout)
        # generate hosts list based on the layout host count
        inven_hosts = self.get_hosts_by_count(host_ip_dict, layout_host_count)
        # adding sections to respective host groups
        host_groups = self.get_layout_host_groups(layout)
        self.add_sections(host_groups)
        # set children for each host group
        self.set_children(layout)
        # set vars for each host group
        self.set_vars(layout)
        # add ip addresses to each host
        self.add_ips_to_groups(inven_hosts, layout)
        self.add_common_vars(host_groups, layout)
        output = StringIO.StringIO()
        self.config.write(output)
        return output.getvalue()



#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import ast
import json

from ansible.module_utils.basic import AnsibleModule

from linchpin.rundb.basedb import BaseDB
from linchpin.rundb.drivers import DB_DRIVERS


# ---- Documentation Start ----------------#
DOCUMENTATION = '''
---
version_added: "0.1"
module: rundb
short_description: A run database for linchpin, though it
                   could be used for anything transactional
                   really.
description:
  - This module allows a user to store and retrive values.
options:
  db_type:
    description:
      Type of Database used
    required: false
    default: TinyRunDB
  conn_str:
    description:
      Connection string to the database, can be file or url
    required: true
  db_schema:
    description:
      Database schema to use when running operations.
      The schema is required for an 'init' operation.
    required: false
  operation:
    description:
      Operation being performed on the database
      Available operations: init, update, purge, search
    required: true
    default: update
  run_id:
    description:
      The run_id value describes the current transaction.
      The run_id is returned for an 'init' operation.
      The run_id is required for an 'update' operation.
    required: false
    default: None
  table: (aka target in linchpin)
    description:
      Name of the table to update in the database
    required: true
  action:
    description:
      Action being performed, up/destroy
    required: false
    default: up
  key:
    description:
      key for the db record
    required: true (except for init and purge operations)
  value:
    description:
      value for the db record. Must be a dictionary of values.
    required: true (except for init and purge operations)

example


requirements: [TinyDB (or your own database)]
author: Clint Savage - herlo@redhat.com
'''


def main():

    module = AnsibleModule(
        argument_spec=dict(
            db_type=dict(type='str', required=False, default='TinyRunDB'),
            conn_str=dict(type='str', required=True),
            operation=dict(choices=['init',
                                    'update',
                                    'purge',
                                    'get'],
                           default='update'),
            run_id=dict(type='int', required=False),
            table=dict(type='str', required=True),
            action=dict(type='str', required=False, default='up'),
            key=dict(type='str', required=False),
            value=dict(type='str', required=False),
        ),
    )
    db_type = module.params['db_type']
    conn_str = os.path.expanduser(module.params['conn_str'])
    table = module.params['table']
    action = module.params['action']
    op = module.params['operation']
    key = module.params['key']
    value = module.params['value']
    run_id = module.params['run_id']

    is_changed = False
    output = None
    try:
        rundb = BaseDB(DB_DRIVERS[db_type], conn_str=conn_str)

        val = value
        # if value looks like a dict it is a dict!
        if value and (value.startswith("{") or value.startswith("[")):
            try:
                val = json.loads(value)
            except Exception as e:
                try:
                    val = ast.literal_eval(value)
                except Exception as e:
                    module.fail_json(msg=e)

        if op in ['init', 'purge']:
            if op == "init":
                if val:
                    rundb.schema = val
                    output = rundb.init_table(table)
                else:
                    msg = ("'table' and 'value' required for init operation")
                    module.fail_json(msg=msg)
            if op == "purge":
                output = rundb.purge(table=table)

        elif op in ['update', 'get']:
            if op == "update":
                if run_id and key and val:
                    runid = int(run_id)
                    output = rundb.update_record(table, runid, key, val)[0]
                else:
                    msg = ("'table', 'run_id, 'key', and 'value' required"
                           " for update operation")
                    module.fail_json(msg=msg)

            if op == "get":
                if run_id and key:
                    runid = int(run_id)
                    record = rundb.get_record(table,
                                              action=action,
                                              run_id=runid)[0]

                    if key in record:
                        output = record.get(key)
                    else:
                        msg = "key '{0}' was not found in record".format(key)
                        module.fail_json(msg=msg)

        else:
            msg = ("Module 'action' required".format(action))
            module.fail_json(msg=msg)

        if output:
            is_changed = True

        module.exit_json(output=output, changed=is_changed)

    except Exception as e:
        module.fail_json(msg=str(e))


#
#            module.exit_json(output=output, changed=output)
#        except Exception as e:
#            module.fail_json(msg=str(e))


# ---- Import Ansible Utilities (Ansible Framework) -------------------#
main()



"""
Inventory Providers List
"""
from linchpin.exceptions import LinchpinError

from AWSInventory import AWSInventory
from BeakerInventory import BeakerInventory
from DuffyInventory import DuffyInventory
from DummyInventory import DummyInventory
from GCloudInventory import GCloudInventory
from LibvirtInventory import LibvirtInventory
from OpenstackInventory import OpenstackInventory
from OvirtInventory import OvirtInventory

filter_classes = {
    "aws_inv": AWSInventory,
    "beaker_inv": BeakerInventory,
    "duffy_inv": DuffyInventory,
    "dummy_inv": DummyInventory,
    "gcloud_inv": GCloudInventory,
    "libvirt_inv": LibvirtInventory,
    "os_inv": OpenstackInventory,
    "ovirt_inv": OvirtInventory,
}


def get_driver(provider):

    if provider not in filter_classes:
            raise LinchpinError("Key {0} not found in"
                                " dictionary".format(provider))

    return filter_classes[provider]


def get_all_drivers():
    return filter_classes



import os
import urlparse
import tempfile

from fetch import Fetch
from linchpin.exceptions import LinchpinError


class FetchLocal(Fetch):

    def __init__(self, ctx, fetch_type, src, dest, cache_dir, root):
        super(FetchLocal, self).__init__(ctx, fetch_type, dest, root)

        self.cache_dir = os.path.join(cache_dir, "local")
        if not os.path.exists(self.cache_dir):
            os.mkdir(self.cache_dir)

        src_parse = urlparse.urlparse(src)
        self.src = os.path.abspath(os.path.join(
            src_parse.netloc, src_parse.path))

        if not os.path.exists(self.src):
            raise LinchpinError('{0} is not a valid path'.format(src))
        if os.path.samefile(self.src, self.dest):
            raise LinchpinError("Provide two different locations")


    def fetch_files(self):
        paths = []
        if self.root is not None:
            for ext in self.root:
                paths.append(os.path.join(self.src, ext.lstrip('/')))
        else:
            paths.append(self.src)

        for path in paths:
            key = "{0}|{1}".format(self.dest.replace(':', ''),
                                   path.replace(':', ''))
            fetch_dir = self.cfgs["local"].get(key, None)
            if not fetch_dir:
                td = tempfile.mkdtemp(prefix="local_", dir=self.cache_dir)
                self.write_cfg("local", key, td)
            else:
                td = fetch_dir

            self.copy_dir(path, td)
            self.tempdirs.append(td)



#!/usr/bin/env python

# flake8: noqa

from ansible import errors


def compare_host_counts(topo, inv):
    t_host_count = 0
    i_host_count = 0

    for k, v in inv['hosts'].iteritems():
        i_host_count += v.get('count', 1)

    t_host_count = len(topo['hosts'])

    return t_host_count >= i_host_count


def make_host_vars(host, host_vars):
    hv = {}

    rep = {"__IP__": host}

    for k, v in host_vars.iteritems():
        for i, j in rep.iteritems():
            v = v.replace(i, j)
        hv.update({k: v})

    return hv


def get_groups(topo, inv):
    # get the number left of hosts from the outputs
    t_count = len(topo['hosts'])
    t_prev_count = t_count
    inv_groups = {}
    hv = inv.get('vars')

    for k, v in inv['hosts'].iteritems():
        num_needed = v.get('count', 1)
        if t_count > 0:
            t_count -= num_needed

            for c in range(t_count, t_prev_count):

                for hg in v.get('host_groups'):
                    host = topo['hosts'][c]
                    host_vars = make_host_vars(host, hv)
                    if hg in inv_groups.keys():
                        inv_groups[hg][host] = host_vars
                    else:
                        inv_groups[hg] = {host: host_vars}
            t_prev_count = t_count

    return inv_groups


def get_group_children(inv):

    hg = inv.get('host_groups')
    inventory = ''

    for k, v in hg.iteritems():
        g_kids = v.get('children')
        if g_kids is not None:
            inventory += '[{0}:children]\n'.format(k)
            for i in g_kids:
                inventory += '{0}\n'.format(i)

    return inventory


def get_group_vars(inv):

    hg = inv.get('host_groups')
    inventory = ''

    for k, v in hg.iteritems():
        g_vars = v.get('vars')
        if g_vars is not None:
            inventory += '[{0}:vars]\n'.format(k)
            for i, j in g_vars.iteritems():
                inventory += '{0}={1}'.format(i, j)
                if not isinstance(j, basestring) or not j.endswith('\n'):
                    inventory += '\n'
    return inventory


def duffy_inventory(topo, inv):

    print(topo['duffy_res'])
    if len(topo['duffy_res']):
        d_topo = topo['duffy_res'][0]
    else:
        return ''
    inventory = ''
    # verify counts. If there aren't enough nodes allocated, abort!
    if compare_host_counts(d_topo, inv):
        groups = get_groups(d_topo, inv)
        g_vars = get_group_vars(inv)
        g_kids = get_group_children(inv)
    for k, v in groups.iteritems():
        inventory += '[{0}]\n'.format(k)
        for i, j in v.iteritems():
            hv = ''
            for k, l in j.iteritems():
                hv += ' {0}={1}'.format(k, l)

            inventory += '{0}{1}\n'.format(i, hv)
        inventory += '\n'
    if g_vars is not None:
        inventory += '{0}\n'.format(g_vars)
    if g_kids is not None:
        inventory += '{1}\n'.format(g_kids)
    inventory += '\n'
    return inventory


class FilterModule(object):
    ''' A filter to fix interface's name format '''
    def filters(self):
        return {
            'duffy_inventory': duffy_inventory
        }



#!/usr/bin/env python

import os
import logging

try:
    import configparser as ConfigParser
except ImportError:
    import ConfigParser as ConfigParser

from linchpin.exceptions import LinchpinError
from linchpin.version import __version__


class LinchpinContext(object):

    """
    LinchpinContext object, which will be used to manage the cli,
    and load the configuration file.
    """


    def __init__(self):
        """
        Initializes basic variables
        """

        self.version = __version__
        self.verbose = False

        lib_path = '{0}'.format(os.path.dirname(
            os.path.realpath(__file__))).rstrip('/')
        self.lib_path = os.path.realpath(os.path.join(lib_path, os.pardir))

        self.cfgs = {}


    def load_config(self, lpconfig=None):
        """
        Create self.cfgs from the linchpin configuration file.

        .. note:: Overrides load_config in linchpin.api.LinchpinContext

        These are the only hardcoded values, which are used to find the config
        file. The search path consists of the following::

          * /linchpin/library/path/linchpin.conf
          * /etc/linchpin.conf
          * ~/.config/linchpin/linchpin.conf
          * path/to/workspace/linchpin.conf

        Linchpin will continuously override and extend the configuration as
        newer configurations are added and modified. Alternatively, a full path
        to the linchpin configuration file can be passed.

        :param lpconfig: absolute path to a linchpin config (default: None)

        """

        expanded_path = None

        if lpconfig:
            CONFIG_PATH = [lpconfig]
        else:
            # simply modify this variable to adjust where
            # linchpin.conf can be found
            CONFIG_PATH = [
                '{0}/linchpin.conf'.format(self.lib_path),
                '/etc/linchpin.conf',
                '~/.config/linchpin/linchpin.conf',
                '{0}/linchpin.conf'.format(self.workspace)
                # self.workspace is set in runcli beforehand, will never be None
            ]
        existing_paths = []
        for path in CONFIG_PATH:
            expanded_path = (
                "{0}".format(os.path.realpath(os.path.expanduser(path))))

            # implement first found
            if os.path.exists(expanded_path):
                # logging before the config file is setup doesn't work
                # if messages are needed before this, use print.
                existing_paths.append(expanded_path)
        if len(existing_paths) == 0:
            raise LinchpinError('Configuration file not found in'
                                ' path: {0}'.format(CONFIG_PATH))
        try:
            for path in existing_paths:
                config = ConfigParser.SafeConfigParser()
                f = open(path)
                config.readfp(f)
                f.close()

                for section in config.sections():
                    self.cfgs[section] = {}
                    for k, v in config.items(section):
                        if section != 'evars':
                            self.cfgs[section][k] = v
                        else:
                            try:
                                self.cfgs[section][k] = (
                                    config.getboolean(section, k)
                                )
                            except ValueError as e:
                                self.cfgs[section][k] = v
        except ConfigParser.InterpolationSyntaxError as e:
            raise LinchpinError('Unable to parse configuration file properly:'
                                ' {0}'.format(e))

    def load_global_evars(self):

        """
        Instantiate the evars variable, then load the variables from the
        'evars' section in linchpin.conf. This will then be passed to
        invoke_linchpin, which passes them to the Ansible playbook as needed.

        """

        self.evars = self.cfgs.get('evars', {})


    def get_cfg(self, section=None, key=None, default=None):
        """
        Get cfgs value(s) by section and/or key, or the whole cfgs object

        :param section: section from ini-style config file

        :param key: key to get from config file, within section

        :param default: default value to return if nothing is found.
        Does not apply if section is not provided.
        """

        if section:
            s = self.cfgs.get(section, default)
            if key and s:
                return self.cfgs[section].get(key, default)
            return s
        return self.cfgs


    def set_cfg(self, section, key, value):
        """
        Set a value in cfgs. Does not persist into a file,
        only during the current execution.

        :param section: section within ini-style config file
        :param key: key to use
        :param value: value to set into section within config file
        """

        if not self.cfgs.get(section):
            self.cfgs.update({section: {}})

        self.cfgs[section][key] = value


    def get_evar(self, key=None, default=None):
        """
        Get the current evars (extra_vars)

        :param key: key to use

        :param default: default value to return if nothing is found
        (default: None)
        """

        if key:
            return self.evars.get(key, default)

        return self.evars


    def set_evar(self, key, value):
        """
        Set a value into evars (extra_vars). Does not persist into a file,
        only during the current execution.

        :param key: key to use

        :param value: value to set into evars
        """

        self.set_cfg('evars', key, value)


    @property
    def pinfile(self):

        """
        getter function for pinfile name
        """

        return self.get_cfg('init', 'pinfile')


    @pinfile.setter
    def pinfile(self, pinfile):

        """
        setter for workspace
        """

        self.set_cfg('init', 'pinfile', pinfile)


    @property
    def workspace(self):

        """
        getter function for workspace
        """

        return self.get_cfg('lp', 'workspace')


    @workspace.setter
    def workspace(self, workspace):

        """
        setter for workspace
        """

        self.set_cfg('lp', 'workspace', workspace)


    def setup_logging(self):

        """
        Setup logging to the console only

        .. attention:: Please implement this function in a subclass

        """

        self.console = logging.getLogger('lp_console')
        self.console.setLevel(logging.INFO)

        ch = logging.StreamHandler()
        ch.setLevel(logging.INFO)

        formatter = logging.Formatter('%(levelname)s %(asctime)s %(message)s')
        ch.setFormatter(formatter)

        self.console.addHandler(ch)


    def log(self, msg, **kwargs):
        """
        Logs a message to a logfile

        :param msg: message to output to log

        :param level: keyword argument defining the log level

        """

        lvl = kwargs.get('level')

        if lvl is None:
            lvl = logging.INFO

        self.console.log(logging.INFO, msg)


    def log_info(self, msg):
        """Logs an INFO message"""

        self.log(msg, level=logging.INFO)


    def log_debug(self, msg):
        """Logs a DEBUG message"""

        self.log(msg, level=logging.DEBUG)


    def log_state(self, msg):
        """
        Logs nothing, just calls pass

        .. attention:: state messages need to be implemented in a subclass
        """

        pass



#!/usr/bin/env python


def format_networks(networks):
    # "net-name=atomic-e2e-jenkins-test,net-name=atomic-e2e-jenkins-test2"
    nics = ["net-name={0}".format(net) for net in networks]
    nics = ",".join(nics)
    return str(nics)


class FilterModule(object):
    ''' A filter to fix network format '''
    def filters(self):
        return {
            'os_net': format_networks
        }



#!/usr/bin/python
# -*- coding: utf-8 -*-

# (c) 2012, Michael DeHaan <michael.dehaan@gmail.com>, and others
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
import os
import json

from ansible.module_utils.basic import AnsibleModule
DOCUMENTATION = '''
---
module: async_status
short_description: Obtain status of asynchronous task
description:
     - "This module gets the status of an asynchronous task."
version_added: "0.5"
options:
  jid:
    description:
      - Job or task identifier
    required: true
    default: null
    aliases: []
  mode:
    description:
      - if C(status), obtain the status:
        if C(cleanup), clean up the async job cache
        located in C(~/.ansible_async/) for the specified job I(jid).
    required: false
    choices: [ "status", "cleanup" ]
    default: "status"
notes:
    - See also U(http://docs.ansible.com/playbooks_async.html)
requirements: []
author:
    - "Ansible Core Team"
    - "Michael DeHaan"
'''


def main():

    module = AnsibleModule(argument_spec=dict(
        jid=dict(required=True),
        mode=dict(default='status', choices=['status', 'cleanup']),
        module_name=dict(default='', choices=['os_server', 'os_volume', '']),
    ))

    mode = module.params['mode']
    jid = module.params['jid']
    module_name = module.params['module_name']

    # setup logging directory
    logdir = os.path.expanduser("~/.ansible_async")
    log_path = os.path.join(logdir, jid)

    if not os.path.exists(log_path):
        module.fail_json(msg="could not find job",
                         ansible_job_id=jid,
                         started=1,
                         finished=1)

    if mode == 'cleanup':
        os.unlink(log_path)
        module.exit_json(ansible_job_id=jid, erased=log_path)

    # NOT in cleanup mode, assume regular status mode
    # no remote kill mode currently exists, but probably should
    # consider log_path + ".pid" file and also unlink that above

    data = None
    try:
        data = file(log_path).read()
        if ((module_name == 'os_server' or module_name == 'os_volume') and
                (len(data.split("\n")) == 4)):
            data = open(log_path).readlines()
            data = data[2]

        data = json.loads(data)
        # data = ast.literal_eval(data)
    except Exception:
        if not data:
            # file not written yet?  That means it is running
            module.exit_json(results_file=log_path,
                             ansible_job_id=jid,
                             started=1,
                             finished=0)
        else:
            module.fail_json(ansible_job_id=jid, results_file=log_path,
                             msg="Could not parse job output: %s" % data,
                             started=1,
                             finished=1)

    if 'started' not in data:
        data['finished'] = 1
        data['ansible_job_id'] = jid
    elif 'finished' not in data:
        data['finished'] = 0

    # Fix error: TypeError: exit_json() keywords must be strings
    data = dict([(str(k), v) for k, v in data.iteritems()])

    module.exit_json(**data)


main()



#!/usr/bin/env python


def format_rules(rules, rule_type):
    rules_output = []
    for rule in rules:
        if rule["rule_type"] == rule_type:
            rule_output = {}
            rule_output['from_port'] = rule['from_port']
            rule_output['to_port'] = rule['to_port']
            rule_output['cidr_ip'] = rule['cidr_ip']
            rule_output['proto'] = rule['proto']
            rules_output.append(rule_output)
    return rules_output


class FilterModule(object):
    ''' A filter to format AWS EC2 security group rules '''
    def filters(self):
        return {
            'aws_sg_rules': format_rules
        }



from tinydb import TinyDB
from tinydb.storages import JSONStorage
from tinydb.operations import add
from tinydb.middlewares import CachingMiddleware

from .basedb import BaseDB


def usedb(func):
    def func_wrapper(*args, **kwargs):
        args[0]._opendb()
        x = func(*args, **kwargs)
        args[0]._closedb()
        return x
    return func_wrapper


class TinyRunDB(BaseDB):

    def __init__(self, conn_str):
        self.name = 'TinyRunDB'
        self.conn_str = conn_str
        self.default_table = 'linchpin'


    def _opendb(self):
        self.middleware = CachingMiddleware(JSONStorage)
        self.middleware.WRITE_CACHE_SIZE = 500
        self.db = TinyDB(self.conn_str, storage=self.middleware,
                         default_table=self.default_table)


    def __str__(self):
        if self.conn_str:
            return "{0} at {2}".format(self.name, self.conn_str)
        return "{0} at {1}".format(self.name, 'None')


    @property
    def schema(self):
        return self._schema


    @schema.setter
    def schema(self, schema):
        self._schema = dict()
        self._schema.update(schema)


    @usedb
    def init_table(self, table):
        t = self.db.table(name=table)
        return t.insert(self.schema)


    @usedb
    def update_record(self, table, run_id, key, value):
        t = self.db.table(name=table)
        return t.update(add(key, value), eids=[run_id])


    @usedb
    def get_record(self, table, action='up', run_id=None):

        t = self.db.table(name=table)
        if not run_id:
            run_id = len(t.all())
            if not run_id:
                return (None, 0)

        for rid in range(int(run_id), 0, -1):
            record = t.get(eid=int(rid))
            if record['action'] == action:
                return (record, int(rid))

        return (None, 0)


    @usedb
    def get_records(self, table, count=10):

        records = {}
        if table in self.db.tables():
            t = self.db.table(name=table)
            start = len(t)
            end = start - count

            for i in xrange(start, end, -1):
                records[i] = t.get(doc_id=i)

        return records


    @usedb
    def get_tables(self):

        tables = self.db.tables()
        tables.remove(self.default_table)

        return tables


    def remove_record(self, table, key, value):
        pass


    def search(self, table, key=None):
        t = self.db.table(name=table)
        if key:
            return t.search(key)
        return t.all()


    def query(self, table, query):
        pass


    def purge(self, table=None):
        if table:
            return self.db.purge_table(table)
        return self.db.purge_tables()


    def _closedb(self):
        self.db.close()



#!/usr/bin/env python

import yaml
import yamlordereddictloader
from camel import Camel


def to_ordered_dict(filepath):
    data = yaml.load(open(filepath), Loader=yamlordereddictloader.Loader)
    data = Camel().dump(data["inventory_layout"])
    return data


class FilterModule(object):
    ''' A filter to fix interface's name format '''
    def filters(self):
        return {
            'ordered_yaml': to_ordered_dict
        }



import os
import subprocess
import tempfile

from fetch import Fetch
from linchpin.exceptions import LinchpinError


class FetchGit(Fetch):
    def __init__(self, ctx, fetch_type, src, dest, cache_dir, root):
        super(FetchGit, self).__init__(ctx, fetch_type, dest, root)
        self.src = src

        self.cache_dir = os.path.join(cache_dir, "git")
        if not os.path.exists(self.cache_dir):
            os.mkdir(self.cache_dir)

    def fetch_files(self):
        # The key cannot contain ':' since linchpin does not support python 3.
        # Configparser uses ':' as a delimiter, which poses problem when using
        # urls in key during parsing. Delimiters can be specified when
        # initializing a configparser object in python 3 so this does not
        # become an issue.
        key = "{0}|{1}".format(self.dest.replace(':', ''),
                               self.src.replace(':', ''))

        fetch_dir = self.cfgs["git"].get(key, None)
        td = self.call_clone(fetch_dir)

        if fetch_dir is None:
            self.write_cfg("git", key, td)

        if self.root is not None:
            for ext in self.root:
                self.tempdirs.append(os.path.join(td, ext.lstrip('/')))
        else:
            self.tempdirs.append(td)


    def call_clone(self, fetch_dir=None):
        if fetch_dir:
            retval = subprocess.call(
                ['git', '-C', fetch_dir, 'pull', '--quiet'])
            tempdir = fetch_dir
        else:
            tempdir = tempfile.mkdtemp(prefix="git_", dir=self.cache_dir)
            retval = subprocess.call(
                ['git', 'clone', '--quiet', self.src, tempdir])

        if retval != 0:
            raise LinchpinError("Unable to clone {0}".format(self.src))
        return tempdir



#!/usr/bin/env python

import ast
import logging

from linchpin.api.context import LinchpinContext


class LinchpinCliContext(LinchpinContext):
    """
    Context object, which will be used to manage the cli,
    and load the configuration file
    """


    def __init__(self):
        """
        Initializes basic variables
        """

        # The following values are set in the parent class
        #
        # self.version = __version__
        # self.verbose = False
        #
        # lib_path = '{0}'.format(os.path.dirname(
        #                       os.path.realpath(__file__))).rstrip('/')
        # self.lib_path = os.path.realpath(os.path.join(lib_path, os.pardir))
        #
        # self.cfgs = {}

        LinchpinContext.__init__(self)


    def load_config(self, lpconfig=None):
        return super(LinchpinCliContext, self).load_config(lpconfig)

    def setup_logging(self):

        """
        Setup logging to a file, console, or both. Modifying the `linchpin.conf`
        appropriately will provide functionality.

        """

        self.enable_logging = ast.literal_eval(
            self.cfgs['logger'].get('enable', 'True'))

        if self.enable_logging:

            # create logger
            self.logger = logging.getLogger('lp_logger')
            self.logger.setLevel(eval(self.cfgs['logger'].get('level',
                                                              'logging.DEBUG')))

            fh = logging.FileHandler(self.cfgs['logger'].get('file',
                                                             'linchpin.log'))
            fh.setLevel(eval(self.cfgs['logger'].get('level',
                                                     'logging.DEBUG')))
            formatter = logging.Formatter(
                self.cfgs['logger'].get('format',
                                        '%(levelname)s'
                                        ' %(asctime)s %(message)s'))
            fh.setFormatter(formatter)
            self.logger.addHandler(fh)


        self.console = logging.getLogger('lp_console')
        self.console.setLevel(eval(self.cfgs['console'].get('level',
                                                            'logging.INFO')))

        ch = logging.StreamHandler()
        ch.setLevel(eval(self.cfgs['console'].get('level',
                                                  'logging.INFO')))
        formatter = logging.Formatter(
            self.cfgs['console'].get('format', '%(message)s'))
        ch.setFormatter(formatter)
        self.console.addHandler(ch)


    def log(self, msg, **kwargs):
        """
        Logs a message to a logfile or the console

        :param msg: message to log

        :param lvl: keyword argument defining the log level

        :param msg_type: keyword argument giving more flexibility.

        .. note:: Only msg_type `STATE` is currently implemented.
        """

        lvl = kwargs.get('level')
        msg_type = kwargs.get('msg_type')

        if lvl is None:
            lvl = logging.INFO

        if self.verbose and not msg_type:
            self.console.log(logging.INFO, msg)

        state_msg = msg
        if msg_type == 'STATE':
            state_msg = 'STATE - {0}'.format(msg)
            self.console.log(logging.INFO, msg)

        if self.enable_logging:
            self.logger.log(lvl, state_msg)


    def log_state(self, msg):
        """Logs a message to stdout"""

        self.log(msg, msg_type='STATE', level=logging.DEBUG)

    def log_info(self, msg):
        """Logs an INFO message """
        self.log(msg, level=logging.INFO)

    def log_debug(self, msg):
        """Logs a DEBUG message"""
        self.log(msg, level=logging.DEBUG)



#!/usr/bin/env python


def provide_default(fetched, default):
    if fetched == "":
        return default
    else:
        return fetched


class FilterModule(object):
    ''' A filter to fix interface's name format '''
    def filters(self):
        return {
            'provide_default': provide_default
        }



#!/usr/bin/env python

import os
import sys
import json
import click

from linchpin.cli import LinchpinCli
from linchpin.exceptions import LinchpinError
from linchpin.cli.context import LinchpinCliContext


pass_context = click.make_pass_decorator(LinchpinCliContext, ensure=True)
CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])


class LinchpinAliases(click.Group):

    lp_commands = ['init', 'up', 'destroy', 'fetch', 'journal']
    lp_aliases = {
        'rise': 'up',
        'drop': 'destroy',
        'down': 'destroy',
    }

    def list_commands(self, ctx):
        """
        Provide a list of available commands. Anhthing deprecated should
        not be listed
        """

        return self.lp_commands

    def get_command(self, ctx, name):

        """
        Track aliases for specific commands and the commands and return the
        correct action.
        """

        cmd = self.lp_aliases.get(name)

        if cmd is None:
            cmd = name

        rv = click.Group.get_command(self, ctx, cmd)
        return rv


def _handle_results(ctx, results, return_code):
    """
    Handle results from the RunDB output and Ansible API.
    Either as a return value (retval) when running with the
    ansible console enabled, or as a list of TaskResult
    objects, and a return value.

    If a target fails along the way, this method immediately exits with the
    appropriate return value (retval). If the ansible console is disabled, an
    error message will be printed before exiting.

    :param results:
        The dictionary of results for each target.
    """

    output = '\n{0:<20}\t{1:<6}\t{2:<5}\t{3:<10}\n'.format('Target',
                                                           'Run ID',
                                                           'uHash',
                                                           'Exit Code')
    output += '-------------------------------------------------\n'

    for target, data in results.iteritems():
        rundb_data = data['rundb_data']
        task_results = data['task_results']

        if not isinstance(task_results, int):
            trs = task_results

            if trs is not None:
                trs.reverse()
                tr = trs[0]
                if tr.is_failed():
                    msg = tr._check_key('msg')
                    ctx.log_state("Target '{0}': {1} failed with"
                                  " error '{2}'".format(target, tr._task, msg))
        else:
            if task_results:
                return_code = task_results

        # PRINT OUTPUT RESULTS HERE
        for rundb_id, data in rundb_data.iteritems():

            output += '{0:<20}\t{1:>6}\t{2:>5}'.format(target,
                                                       rundb_id,
                                                       data['uhash'])

            output += '\t{0:>9}\n'.format(return_code)


    ctx.log_state(output)
    sys.exit(return_code)


def _get_pinfile_path(pinfile=None, exists=True):

    if not pinfile:
        pinfile = lpcli.pinfile

    pf_w_path = '{0}/{1}'.format(lpcli.workspace, pinfile)

    if not os.path.exists(pf_w_path) and exists:
        lpcli.ctx.log_state('{0} not found in provided workspace: '
                            '{1}'.format(pinfile, lpcli.workspace))
        sys.exit(1)

    return pf_w_path


@click.group(cls=LinchpinAliases,
             invoke_without_command=True,
             no_args_is_help=True,
             context_settings=CONTEXT_SETTINGS)
@click.option('-c', '--config', type=click.Path(), envvar='LP_CONFIG',
              help='Path to config file')
@click.option('-p', '--pinfile', envvar='PINFILE',
              help='Use a name for the PinFile different from'
                   ' the configuration.')
@click.option('-w', '--workspace', type=click.Path(), envvar='WORKSPACE',
              help='Use the specified workspace if the familiar Jenkins'
                   ' $WORKSPACE environment variable is not set')
@click.option('-v', '--verbose', is_flag=True, default=False,
              help='Enable verbose output')
@click.option('--version', is_flag=True,
              help='Prints the version and exits')
@click.option('--creds-path', type=click.Path(), envvar='CREDS_PATH',
              help='Use the specified credentials path if CREDS_PATH'
                   'environment variable is not set')
@pass_context
def runcli(ctx, config, pinfile, workspace, verbose, version, creds_path):
    """linchpin: hybrid cloud orchestration"""

    ctx.load_config(lpconfig=config)
    # workspace arg in load_config used to extend linchpin.conf
    ctx.load_global_evars()
    ctx.setup_logging()

    ctx.verbose = verbose

    if pinfile:
        ctx.log_info('pinfile name changed to {0}'.format(pinfile))
        ctx.pinfile = pinfile

    if version:
        ctx.log_state('linchpin version {0}'.format(ctx.version))
        sys.exit(0)

    if creds_path is not None:
        ctx.set_evar('creds_path',
                     os.path.realpath(os.path.expanduser(creds_path)))

    if workspace is not None:
        ctx.workspace = os.path.realpath(os.path.expanduser(workspace))
        ctx.log_debug("ctx.workspace: {0}".format(ctx.workspace))

    # global LinchpinCli placeholder
    global lpcli
    lpcli = LinchpinCli(ctx)


@runcli.command('init', short_help='Initializes a linchpin project.')
@pass_context
def init(ctx):
    """
    Initializes a linchpin project, which generates an example PinFile, and
    creates the necessary directory structure for topologies and layouts.

    ctx: Context object defined by the click.make_pass_decorator method
    """

    pf_w_path = _get_pinfile_path(exists=False)

    try:
        # lpcli.lp_init(pf_w_path, targets) # TODO implement targets option
        lpcli.lp_init(pf_w_path)
    except LinchpinError as e:
        ctx.log_state(e)
        sys.exit(1)


@runcli.command()
@click.argument('targets', metavar='TARGETS', required=False,
                nargs=-1)
@click.option('-r', '--run-id', metavar='run_id',
              help='Idempotently provision using `run-id` data')
@pass_context
def up(ctx, targets, run_id):
    """
    Provisions nodes from the given target(s) in the given PinFile.

    pinfile:    Path to pinfile (Default: workspace path)

    targets:    Provision ONLY the listed target(s). If omitted, ALL targets in
    the appropriate PinFile will be provisioned.
    """

    pf_w_path = _get_pinfile_path()

    try:
        return_code, results = lpcli.lp_up(pf_w_path, targets, run_id=run_id)
        _handle_results(ctx, results, return_code)

    except LinchpinError as e:
        ctx.log_state(e)
        sys.exit(1)


@runcli.command()
@click.argument('targets', metavar='TARGET', required=False, nargs=-1)
@pass_context
def rise(ctx, targets):
    """
    DEPRECATED. Use 'up'

    """

    pass


@runcli.command()
@click.argument('targets', metavar='TARGET', required=False,
                nargs=-1)
@click.option('-r', '--run-id', metavar='run_id',
              help='Destroy resources using `run-id` data')
@pass_context
def destroy(ctx, targets, run_id):
    """
    Destroys nodes from the given target(s) in the given PinFile.

    pinfile:    Path to pinfile (Default: workspace path)

    targets:    Destroy ONLY the listed target(s). If omitted, ALL targets in
    the appropriate PinFile will be destroyed.

    """

    pf_w_path = _get_pinfile_path()

    try:
        return_code, results = lpcli.lp_destroy(pf_w_path,
                                                targets,
                                                run_id=run_id)

        _handle_results(ctx, results, return_code)

    except LinchpinError as e:
        ctx.log_state(e)
        sys.exit(1)


@runcli.command()
@click.argument('targets', metavar='TARGET', required=False,
                nargs=-1)
@pass_context
def drop(ctx, targets):
    """
    DEPRECATED. Use 'destroy'.

    There are now two functions, `destroy` and `down` which perform node
    teardown. The `destroy` functionality is the default, and if drop is
    used, will be called.

    The `down` functionality is currently unimplemented, but will shutdown
    and preserve instances. This feature will only work on providers that
    support this option.

    """

    pass


@runcli.command()
@click.argument('fetch_type', default=None, required=False, nargs=-1)
@click.argument('remote', default=None, required=True, nargs=1)
@click.option('-r', '--root', default=None, required=False,
              help='Use this to specify the subdirectory of the workspace of'
              ' the root url')
@pass_context
def fetch(ctx, fetch_type, remote, root):
    """
    Fetches a specified linchpin workspace or component from a remote location.

    fetch_type:     Specifies which component of a workspace the user wants to
    fetch. Types include: topology, layout, resources, hooks, workspace

    remote:         The URL or URI of the remote directory

    """
    try:
        lpcli.lp_fetch(remote, ''.join(fetch_type), root)
    except LinchpinError as e:
        ctx.log_state(e)
        sys.exit(1)


@runcli.command()
@click.argument('targets', metavar='TARGETS', required=True, nargs=-1)
@click.option('-c', '--count', metavar='COUNT', default=3, required=False,
              help='(up to) number of records to return (default: 3)')
@click.option('-f', '--fields', metavar='FIELDS', required=False,
              help='List the fields to display')
@pass_context
def journal(ctx, targets, fields, count):
    """
    Display information stored in Run Database

    targets:    Display data for the listed target(s). If omitted, the latest
                records for any/all targets in the RunDB will be displayed.

    fields:     Comma separated list of fields to show in the display.
    (Default: action, uhash, rc)

    (available fields are: uhash, rc, start, end, action)

    """

    all_fields = json.loads(lpcli.get_cfg('lp', 'rundb_schema')).keys()

    if not fields:
        fields = ['action', 'uhash', 'rc']
    else:
        fields = fields.split(',')

    invalid_fields = [field for field in fields if field not in all_fields]

    if invalid_fields:
        ctx.log_state('The following fields passed in are not valid: {0}'
                      ' \nValid fields are {1}'.format(fields, all_fields))
        sys.exit(89)

    no_records = []
    output = 'run_id\t'

    for f in fields:
        output += '{0:>10}\t'.format(f)

    output += '\n'
    output += '--------------------------------------------------'


    try:
        journal = lpcli.lp_journal(targets=targets, fields=fields, count=count)

        for target, values in journal.iteritems():

            keys = values.keys()
            if len(keys):
                print('\nTarget: {0}'.format(target))
                print(output)
                keys.sort(reverse=True)
                for run_id in keys:
                    if int(run_id) > 0:
                        out = '{0:<7}\t'.format(run_id)
                        for f in fields:
                            out += '{0:>9}\t'.format(values[run_id][f])

                        print(out)
            else:
                no_records.append(target)

        if no_records:
            no_out = '\nNo records for targets:'

            for rec in no_records:
                no_out += ' {0}'.format(rec)

            no_out += '\n'

            print(no_out)

    except LinchpinError as e:
        ctx.log_state(e)
        sys.exit(1)


def main():
    # print("entrypoint")
    pass



#!/usr/bin/env python

import StringIO

from InventoryFilter import InventoryFilter


class OpenstackInventory(InventoryFilter):

    def get_host_ips(self, topo):
        host_public_ips = []
        for group in topo['os_server_res']:
            grp = group.get('openstack', [])
            if isinstance(grp, list):
                for server in grp:
                    host_public_ips.append(str(server['accessIPv4']))
            if isinstance(grp, dict):
                host_public_ips.append(str(grp['accessIPv4']))
        return host_public_ips

    def get_inventory(self, topo, layout):

        if len(topo['os_server_res']) == 0:
            return ""
        inven_hosts = self.get_host_ips(topo)
        # adding sections to respective host groups
        host_groups = self.get_layout_host_groups(layout)
        self.add_sections(host_groups)
        # set children for each host group
        self.set_children(layout)
        # set vars for each host group
        self.set_vars(layout)
        # add ip addresses to each host
        self.add_ips_to_groups(inven_hosts, layout)
        self.add_common_vars(host_groups, layout)
        output = StringIO.StringIO()
        self.config.write(output)
        return output.getvalue()




class LinchpinError(Exception):

    def __init__(self, *args, **kwargs):
        Exception.__init__(self, *args, **kwargs)


class HookError(LinchpinError):

    def __init__(self, *args, **kwargs):
        LinchpinError.__init__(self, *args, **kwargs)


class StateError(LinchpinError):

    def __init__(self, *args, **kwargs):
        LinchpinError.__init__(self, *args, **kwargs)


class ActionManagerError(LinchpinError):

    def __init__(self, *args, **kwargs):
        LinchpinError.__init__(self, *args, **kwargs)



import os
import subprocess
import tempfile
import requests

from fetch import Fetch
from linchpin.exceptions import LinchpinError


class FetchHttp(Fetch):

    def __init__(self, ctx, fetch_type, src, dest, cache_dir, root):
        super(FetchHttp, self).__init__(ctx, fetch_type, dest, root)

        self.cache_dir = os.path.join(cache_dir, "http")
        if not os.path.exists(self.cache_dir):
            os.mkdir(self.cache_dir)

        try:
            r = requests.get(src)
            r.raise_for_status()
        except requests.exceptions.HTTPError:
            raise LinchpinError("An HTTP error occurred")
        except requests.exceptions.RequestException:
            raise LinchpinError("Could not connect to given URL")

        if r.status_code != 200:
            raise LinchpinError("The entered url is invalid")

        self.src = src.rstrip('/')

    def fetch_files(self):
        if self.root is not None:
            for ext in self.root:
                src = os.path.join(self.src, ext.lstrip('/'))
                key = "{0}|{1}".format(self.dest.replace(':', ''),
                                       src.replace(':', ''))
                fetch_dir = self.cfgs["http"].get(key, None)

                td = self.call_wget(src, fetch_dir)
                self.tempdirs.append(td)

                if not fetch_dir:
                    self.write_cfg("http", key, td)
        else:
            key = "{0}|{1}".format(self.dest.replace(':', ''),
                                   self.src.replace(':', ''))
            fetch_dir = self.cfgs["http"].get(key, None)

            td = self.call_wget(self.src, fetch_dir)
            self.tempdirs.append(td)

            if not fetch_dir:
                self.write_cfg("http", key, td)

    def call_wget(self, src, fetch_dir=None):
        list_args = src.split('/')
        list_args = list_args[3:]
        tempdir = None

        if fetch_dir is None:
            tempdir = tempfile.mkdtemp(prefix="http_", dir=self.cache_dir)
            wget_args = ['wget', '-r', '-np', '-nH', '-q', '--reject', 'html',
                         '--cut-dirs={0}'.format(len(list_args)),
                         src, '-P', tempdir]
        else:
            tempdir = fetch_dir
            wget_args = ['wget', '-r', '-np', '-N', '-nH', '-q', '--reject',
                         'html', '--cut-dirs={0}'.format(len(list_args)), src,
                         '-P', tempdir]

        retval = subprocess.call(wget_args)

        if retval != 0:
            try:
                os.rmdir(tempdir)
            except OSError:
                pass
            raise LinchpinError('Unable to fetch files with the following'
                                ' command:\n{0}'.format(" ".join(wget_args)))
        return tempdir



from fetch_local import FetchLocal
from fetch_http import FetchHttp
from fetch_git import FetchGit

FETCH_CLASS = {
    "FetchLocal": FetchLocal,
    "FetchHttp": FetchHttp,
    "FetchGit": FetchGit
}
__all__ = ["FetchLocal", "FetchHttp", "FetchGit"]



#!/usr/bin/env python

import os
import sys
import click
from distutils import dir_util

from linchpin.api import LinchpinAPI


class LinchpinCli(LinchpinAPI):

    def __init__(self, ctx):
        """
        Set some variables, pass to parent class
        """

        LinchpinAPI.__init__(self, ctx)


    def lp_init(self, pf_w_path, providers=['libvirt']):
        """
        Initializes a linchpin project. Creates the necessary directory
        structure, includes PinFile, topologies and layouts for the given
        provider. (Default: Libvirt. Other providers not yet implemented.)

        :param pf_w_path: Path to where the PinFile might exist. Gets created
        if it doesn't exist.

        :param providers: A list of providers for which templates
        (and a target) will be provided into the workspace.
        NOT YET IMPLEMENTED
        """

        src = self.get_cfg('init', 'source', 'templates/')
        src_w_path = os.path.realpath('{0}/{1}'.format(self.ctx.lib_path, src))

        src_pf = os.path.realpath('{0}.lp_example'.format(pf_w_path))

        try:
            if os.path.exists(pf_w_path):
                if not click.confirm('{0} already exists,'
                                     'overwrite it?'.format(pf_w_path),
                                     default=False):
                    sys.exit(0)

            dir_util.copy_tree(src_w_path, self.workspace, verbose=1)
            os.rename(src_pf, pf_w_path)

            self.ctx.log_state('{0} and file structure created at {1}'.format(
                self.pinfile, self.workspace))
        except Exception as e:
            self.ctx.log_state('Error: {0}'.format(e))
            sys.exit(1)



"""
example Pinfile for reference::
---
openstack:
  topology: ex_os_server.yml
  layout: openshift-3node-cluster.yml
  hooks:
    postup:              # sub-state is specified
        - name: do_something
          type: shell
          actions:
            - samvaran
        - name: manipulate_inventory
          type: shell
          path: /tmp/shellscripts
          actions:
            - thisisshell.sh
        - name: post_up
          type: ansible
          actions:
            - playbook: test_playbook.yaml
              vars: test_var.yaml
              extra_vars: { 'testvar': 'world'}
    preup:              # sub-state is specified
        - name: do_something
          type: shell
          actions:
            - echo ' this is post up operation Hello hai how r u ?'
        - name: build_openshift_cluster
          type: ansible
          actions:
            - playbook: test_playbook.yaml
              vars: test_var.yaml
              extra_vars: { 'testvar': 'world'}
    postdestroy:
        - name: do_something
          type: shell
          actions:
            - echo ' this is post up operation Hello hai how r u ?'
        - name: postdown_task
          type: ansible
          actions:
            - playbook: test_playbook.yaml
              vars: test_var.yaml
              extra_vars: { 'testvar': 'world'}

"""

import ast
import sys

from linchpin.hooks.action_managers import ACTION_MANAGERS
from linchpin.exceptions import ActionManagerError


class ActionBlockRouter(object):
    """
    Proxy pattern implementation for fetching actionmanagers by name
    """

    def __init__(self, name, *args, **kwargs):

        self.__implementation = self.__get_implementation(name)(name,
                                                                *args,
                                                                **kwargs)

    def __getattr__(self, name):

        return getattr(self.__implementation, name)

    def __get_implementation(self, class_name):
        '''
        Fetches implementation of lcass from linchpin.hooks.action_managers
        :param name: action manager class name
        '''

        action_class = ACTION_MANAGERS.get(class_name, None)
        if action_class is None:
            raise ActionManagerError('Action Class {0}'
                                     'not found '.format(class_name))
        return action_class


class LinchpinHooks(object):

    def __init__(self, api):
        """
        LinchpinHooks class constructor
        :param api: LinchpinAPI object
        """

        self.api = api
        self.api.bind_to_hook_state(self.run_hooks)
        self._rundb = None
        self._rundb_id = None

    @property
    def rundb(self):
        return (self._rundb, self._rundb_id)

    @rundb.setter
    def rundb(self, data):
        self._rundb = data[0]
        self._rundb_id = data[1]


    def prepare_ctx_params(self):
        """
        prepares few context parameters based on the current target_data
        that is being set. these parameters are based topology name.
        """

        # resources are no longer stored in a file
        # commenting out now and will remove later
        # when rundb is fully integrated

        # topology_name = self.api.get_evar("topology_name")
        # res_file = '{0}{1}'.format(topology_name,
        #                           self.api.get_cfg('extensions', 'resource'))
        # res_pthT = self.api.target_data['extra_vars']['default_resources_path'] # noqa
        # res_file = '{0}/{1}'.format(res_pthT, res_file)
        # self.api.target_data['extra_vars']['resource_file'] = res_file

        inv_file_tmp = self.api.get_evar("inventory_file")
        self.api.target_data['extra_vars']['inventory_file'] = inv_file_tmp

        inv_dir_tmp = self.api.get_evar("inventory_dir")
        self.api.target_data['extra_vars']['inventory_dir'] = inv_dir_tmp


    def prepare_inv_params(self):

        target = self.api.get_evar('target', default=None)
        action = self.api.get_evar('_action', default='up')

        topology_data = {}
        layout_data = {}
        results_data = {}

        data, run_id = self._rundb.get_record(target,
                                              action=action,
                                              run_id=self._rundb_id)

        inputs = [i for i in data['inputs'] if data['inputs']]
        outputs = [i for i in data['outputs'] if data['outputs']]

        for inp in inputs:
            if 'layout_data' in inp:
                layout_data = inp
            if 'topology_data' in inp:
                topology_data = inp

        for out in outputs:
            if 'resources' in out:
                results_data = out

        return (topology_data, layout_data, results_data)


    def run_hooks(self, state, is_global=False):
        """
        Function to run hook all hooks from Pinfile based on the state
        :param state: hook state (currently, preup, postup,
        predestroy, postdestroy)
        :param is_global: whether the hook is global (can be applied to
        multiple targets)
        """

        hooks_data = self.api.target_data.get('hooks', None)

        # this will replace the above target_data and pull from the rundb
        # run_data = self.prepare_inv_params()
        if str(state) is 'postinv':
            run_data = self.prepare_inv_params()
            return self.run_inventory_gen(run_data)

        if hooks_data and str(state) in hooks_data:
            self.api.ctx.log_debug('running {0} hooks'.format(state))

            self.prepare_ctx_params()

            # fetches all the state_data , ie., all the action blocks inside
            # state of the target

            state_data = hooks_data.get(str(state), None)

            # Print out error message if the hooks are not found
            if state_data is None:
                self.api.ctx.log_state('{0} hook not found'
                                       ' in PinFile'.format(state))
                return

            # current target data extravars are fetched
            tgt_data = self.api.target_data.get('extra_vars', None)
            self.run_actions(state_data, tgt_data)


    def run_inventory_gen(self, data):
        # determine provisioned resources

        # map outputted resources to inventory layout

        # save to file
        pass


    def run_actions(self, action_blocks, tgt_data, is_global=False):
        """
        Runs actions inside each action block of each target

        :param action_blocks: list of action_blocks each block constitues
                       to a type of hook
        :param tgt_data: data specific to target, which can be dict of
        topology , layout, outputs, inventory
        :param is_global: scope of the hook

        example: action_block:
        - name: do_something
          type: shell
          actions:
            - echo ' this is 'postup' operation Hello hai how r u ?'
        """

        if is_global:
            raise NotImplementedError('Run Hooks is not implemented \
                                       for global scoped hooks')
        else:
            # a_b -> abbr for action_block
            for a_b in action_blocks:
                action_type = a_b['type']
                ab_ctx = a_b['context'] if 'context' in a_b else True
                if 'path' not in a_b:
                    # if the path is not defined it defaults to
                    # workspace/hooks/typeofhook/name
                    a_b['path'] = '{0}/{1}/{2}/{3}/'.format(
                        self.api.ctx.workspace,
                        self.api.get_evar('hooks_folder',
                                          default='hooks'),
                        a_b['type'],
                        a_b['name'])

                if 'action_manager' in a_b:
                    # fetches the action object from the path
                    # add path to python path
                    sys.path.append(a_b['path'])
                    # get the module path
                    module_path = '{0}/{1}'.format(a_b['path'],
                                                   a_b['action_manager'])
                    # get module src
                    module_src = open(module_path, 'r').read()
                    # strip .py ext from module path
                    module_path = module_path.strip('.py')
                    # strip .py ext from action_manager
                    a_b['action_manager'] = a_b['action_manager'].strip('.py')
                    # parse the module
                    module_src = ast.parse(module_src)
                    # get all classes inside the class
                    classes = ([node.name
                               for node in ast.walk(module_src)
                               if isinstance(node, ast.ClassDef)])

                    # choose the first name as the class name
                    class_name = classes[0]
                    # import the module with class name, it should work
                    module = __import__(a_b['action_manager'])
                    # get the class
                    class_ = getattr(module, class_name)
                    # a_b_obj is action_block_object
                    a_b_obj = class_(action_type, a_b, tgt_data, context=ab_ctx)
                else:
                    a_b_obj = ActionBlockRouter(action_type,
                                                a_b,
                                                tgt_data,
                                                context=ab_ctx)
                try:
                    self.api.ctx.log_state('-------\n'
                                           'start hook'
                                           ' {0}:{1}'.format(a_b['type'],
                                                             a_b['name']))

                    # validates the class object
                    a_b_obj.validate()
                    # executes the hook
                    a_b_obj.execute()

                    # intentionally using print here
                    self.api.ctx.log_state('end hook {0}:{1}\n-------'.format(
                                           a_b['type'], a_b['name']))
                except Exception as e:
                    self.api.ctx.log_info(str(e))



#!/usr/bin/env python


def filter_groups(resource_group_types, resource_groups, *args):
    output = {}
    for group_type in resource_group_types:
        output[group_type] = []
        for group in resource_groups:
            for arg in args:
                if (arg in group) and (group[arg] == group_type):
                        output[group_type].append(group)
    return output


class FilterModule(object):
    ''' A filter to fix network format '''
    def filters(self):
        return {
            'res_grp_filter': filter_groups
        }



import os
import sys
import ansible
from collections import namedtuple
from contextlib import contextmanager
from callbacks import PlaybookCallback
from ansible.parsing.dataloader import DataLoader
from ansible.executor.playbook_executor import PlaybookExecutor

ansible24 = float(ansible.__version__[0:3]) >= 2.4

if ansible24:
    from ansible.vars.manager import VariableManager
    from ansible.inventory.manager import InventoryManager as Inventory
else:
    from ansible.inventory import Inventory
    from ansible.vars import VariableManager


@contextmanager
def suppress_stdout():
    """
    This context manager provides tooling to make Ansible's Display class
    not output anything when used
    """
    with open(os.devnull, "w") as devnull:
        old_stdout = sys.stdout
        old_stderr = sys.stderr
        sys.stdout = devnull
        sys.stderr = devnull
        try:
            yield
        finally:
            sys.stdout = old_stdout
            sys.stderr = old_stderr


def ansible_runner_2x(playbook_path,
                      module_path,
                      extra_var,
                      inventory_src='localhost',
                      console=True):
    variable_manager = VariableManager()
    loader = DataLoader()
    extra_var["ansible_python_interpreter"] = sys.executable
    variable_manager.extra_vars = extra_var
    inventory = Inventory(loader=loader,
                          variable_manager=variable_manager,
                          host_list=inventory_src)
    passwords = {}
    Options = namedtuple('Options', ['listtags',
                                     'listtasks',
                                     'listhosts',
                                     'syntax',
                                     'connection',
                                     'module_path',
                                     'forks',
                                     'remote_user',
                                     'private_key_file',
                                     'ssh_common_args',
                                     'ssh_extra_args',
                                     'sftp_extra_args',
                                     'scp_extra_args',
                                     'become',
                                     'become_method',
                                     'become_user',
                                     'verbosity',
                                     'check'])
    options = Options(listtags=False,
                      listtasks=False,
                      listhosts=False,
                      syntax=False,
                      connection='ssh',
                      module_path=module_path,
                      forks=100,
                      remote_user='test',
                      private_key_file=None,
                      ssh_common_args=None,
                      ssh_extra_args=None,
                      sftp_extra_args=None,
                      scp_extra_args=None,
                      become=False,
                      become_method='sudo',
                      become_user='root',
                      verbosity=0,
                      check=False)

    pbex = PlaybookExecutor(playbooks=[playbook_path],
                            inventory=inventory,
                            variable_manager=variable_manager,
                            loader=loader,
                            options=options,
                            passwords=passwords)

    return pbex



def ansible_runner_24x(playbook_path,
                       module_path,
                       extra_var,
                       inventory_src,
                       console=True):
    loader = DataLoader()
    extra_var["ansible_python_interpreter"] = sys.executable
    variable_manager = VariableManager(loader=loader)
    variable_manager.extra_vars = extra_var
    inventory = Inventory(loader=loader, sources=[inventory_src])
    variable_manager.set_inventory(inventory)
    passwords = {}
    Options = namedtuple('Options', ['connection',
                                     'module_path',
                                     'forks',
                                     'become',
                                     'become_method',
                                     'become_user',
                                     'check',
                                     'diff',
                                     'listhosts',
                                     'listtasks',
                                     'listtags',
                                     'syntax',
                                     'remote_user',
                                     'private_key_file',
                                     'ssh_common_args',
                                     'ssh_extra_args',
                                     'sftp_extra_args',
                                     'scp_extra_args',
                                     'verbosity',
                                     ])
    options = Options(listtags=False,
                      listtasks=False,
                      listhosts=False,
                      syntax=False,
                      connection='local',
                      module_path=module_path,
                      forks=100,
                      remote_user='test',
                      private_key_file=None,
                      ssh_common_args=None,
                      ssh_extra_args=None,
                      sftp_extra_args=None,
                      scp_extra_args=None,
                      become=False,
                      become_method='sudo',
                      become_user='root',
                      verbosity=4,
                      diff=False,
                      check=False)
    pbex = PlaybookExecutor(playbooks=[playbook_path],
                            inventory=inventory,
                            variable_manager=variable_manager,
                            loader=loader,
                            options=options,
                            passwords=passwords)
    return pbex


def ansible_runner(playbook_path,
                   module_path,
                   extra_var,
                   inventory_src='localhost',
                   console=True):
        """
        Uses the Ansible API code to invoke the specified linchpin playbook
        :param playbook: Which ansible playbook to run (default: 'up')
        :param console: Whether to display the ansible console (default: True)
        """
        if ansible24:
            pbex = ansible_runner_24x(playbook_path,
                                      module_path,
                                      extra_var,
                                      inventory_src,
                                      console)
        else:
            pbex = ansible_runner_2x(playbook_path,
                                     module_path,
                                     extra_var,
                                     inventory_src,
                                     console)

        if not console:
            results = {}
            return_code = 0
            cb = PlaybookCallback()
            with suppress_stdout():
                pbex._tqm._stdout_callback = cb
            return_code = pbex.run()
            results = cb.results
            return return_code, results
        else:
            # the console only returns a return_code
            return_code = pbex.run()
            return return_code, None
